package v2.simpleUi;

import v2.simpleUi.uiDecoration.UiDecoratable;
import v2.simpleUi.uiDecoration.UiDecorator;
import android.content.Context;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.widget.LinearLayout;

public class M_HalfHalf implements ModifierInterface, UiDecoratable {

	private ModifierInterface myLeft;
	private ModifierInterface myRight;
	private UiDecorator myDecorator;

	public M_HalfHalf(ModifierInterface left, ModifierInterface right) {
		myLeft = left;
		myRight = right;
	}

	@Override
	public View getView(Context context) {
		LayoutParams params = new LinearLayout.LayoutParams(
				LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT, 1);
		LinearLayout l = new LinearLayout(context);
		l.setPadding(l.getPaddingLeft(), l.getPaddingTop() + 10,
				l.getPaddingRight(), l.getPaddingBottom());

		if (myDecorator != null) {
			int level = myDecorator.getCurrentLevel();
			myDecorator.decorate(context, l, level + 1,
					UiDecorator.TYPE_CONTAINER);
			myDecorator.setCurrentLevel(level + 1);
		}

		View left = myLeft.getView(context);
		View right = myRight.getView(context);
		left.setLayoutParams(params);
		right.setLayoutParams(params);
		l.addView(left);
		l.addView(right);

		if (myDecorator != null) {
			myDecorator.setCurrentLevel(myDecorator.getCurrentLevel() - 1);
		}

		return l;
	}

	@Override
	public boolean save() {
		return myLeft.save() && myRight.save();
	}

	@Override
	public boolean assignNewDecorator(UiDecorator decorator) {
		myDecorator = decorator;
		boolean l = false;
		boolean r = false;
		if (myLeft instanceof UiDecoratable) {
			l = ((UiDecoratable) myLeft).assignNewDecorator(decorator);
		}
		if (myRight instanceof UiDecoratable) {
			r = ((UiDecoratable) myRight).assignNewDecorator(decorator);
		}
		return l && r;
	}

}
